@@ -32,6 +32,8 @@ from trexmodel.models.datastore.redeem_models import CustomerRedemption
 from trexmodel.models.datastore.model_decorators import model_transactional
 from trexapi.utils.reward_transaction_helper import revert_redemption,\
     revert_transaction
+from trexanalytics.bigquery_upstream_data_config import create_merchant_registered_customer_upstream_for_merchant,\
+    create_registered_customer_upstream_for_system
 
 logger = logging.getLogger('api')
 
@@ -153,6 +155,10 @@ def create_customer():
                                                             merchant_reference_code = customer_data_in_json.get('merchant_reference_code'), 
                                                             password                = customer_data_in_json.get('password'),
                                                             )
+                            
+                            create_merchant_registered_customer_upstream_for_merchant(created_customer)
+                            create_registered_customer_upstream_for_system(created_customer)
+                            
                         logger.debug('created_customer=%s', created_customer)
                     else:
                         logger.warn('Invalid granted outlet key')
