@@ -21,17 +21,19 @@ from trexadmin.libs.http import create_rest_message
 from trexadmin.libs.http import StatusCode
 from trexmodel.models.datastore.merchant_models import Outlet,\
     MerchantAcct, MerchantUser
-from trexapi.forms.reward_api_forms import GiveRewardTransactionForm, RedeemRewardTransactionForm
+from trexapi.forms.reward_api_forms import GiveRewardTransactionForm, RedeemRewardTransactionForm,\
+    PrepaidTopupForm, PrepaidRedeemForm, PointRedeemForm
 from werkzeug.datastructures import ImmutableMultiDict
-import gettext
 from trexmodel.models.datastore.transaction_models import CustomerTransaction
-from trexadmin.libs.app.utils.reward_transaction_helper import create_sales_transaction,\
-    redeem_reward_transaction
+from trexapi.utils.reward_transaction_helper import create_sales_transaction,\
+    redeem_reward_transaction, create_topup_prepaid_transaction
 from trexapi.utils.api_helpers import get_logged_in_api_username
 from trexmodel.models.datastore.voucher_models import MerchantVoucher
 from trexapi import conf
-
-logger = logging.getLogger('api')
+from trexmodel.models.datastore.prepaid_models import PrepaidSettings
+from trexadmin.libs.jinja.common_filters import format_currency_with_currency_label_filter
+from trexadmin.libs.flask.utils.flask_helper import get_merchant_configured_currency_details
+from trexmodel import program_conf
 
 
 reward_api_bp = Blueprint('reward_api_bp', __name__,
@@ -39,10 +41,10 @@ reward_api_bp = Blueprint('reward_api_bp', __name__,
                                  static_folder='static',
                                  url_prefix='/api/v1/reward')
 
-logger = logging.getLogger('api')
+logger = logging.getLogger('debug')
 
 
-@reward_api_bp.route('/<reference_code>', methods=['GET'])
+@reward_api_bp.route('/reference-code/<reference_code>/read', methods=['GET'])
 @auth_token_required
 def read_reward(reference_code):
     logger.debug('reference_code=%s', reference_code)
@@ -63,6 +65,7 @@ def read_reward(reference_code):
                 for k, v in customer.entitled_voucher_summary.items():
                     for redeem_info in v.get('redeem_info_list'):
                         all_voucher_list.append({
+                                            'key'           : k,
                                             'label'         : v.get('label'),
                                             'image_url'     : v.get('image_url'),
                                             'label'         : v.get('label'),
@@ -87,7 +90,7 @@ def read_reward(reference_code):
         return create_rest_message('Reference code is required', status_code=StatusCode.BAD_REQUEST)
     
 
-@reward_api_bp.route('/<reference_code>/give', methods=['POST'])
+@reward_api_bp.route('/reference-code/<reference_code>/give', methods=['POST'])
 @auth_token_required
 @outlet_key_required
 def give_reward(reference_code):
@@ -112,6 +115,7 @@ def give_reward(reference_code):
             invoice_details     = transaction_data_in_json.get('invoice_details')
             transact_datetime   = None
             
+            
             if tax_amount is None:
                 tax_amount = .0
             else:
@@ -125,6 +129,8 @@ def give_reward(reference_code):
             
             db_client = create_db_client(caller_info="givea_reward")
             
+            check_transaction_by_invoice_id = None
+            
             if is_not_empty(invoice_id):
                 with db_client.context():
                     check_transaction_by_invoice_id = CustomerTransaction.get_by_invoice_id(invoice_id)
@@ -138,17 +144,17 @@ def give_reward(reference_code):
                 if merchant_username:
                     try:
                         with db_client.context():
-                            customer = Customer.get_by_reference_code(reference_code)
-                            if customer:
-                                merchant_acct   = customer.registered_merchant_acct
-                                transact_outlet = Outlet.fetch(request.headers.get('x-outlet-key'))
+                            transact_outlet         = Outlet.fetch(request.headers.get('x-outlet-key'))
+                            merchant_acct           = transact_outlet.merchant_acct_entity
+                            customer                = Customer.get_by_reference_code(reference_code, merchant_acct)
+                            
                                 
                             if transact_datetime_in_gmt:
                                 transact_datetime    =  transact_datetime_in_gmt - timedelta(hours=merchant_acct.gmt_hour)
                                 
                                 now                  = datetime.utcnow()
                                 if transact_datetime > now:
-                                    return create_rest_message(gettext('Transact datetime cannot be future'), status_code=StatusCode.BAD_REQUEST)
+                                    return create_rest_message('Transact datetime cannot be future', status_code=StatusCode.BAD_REQUEST)
                             
                             
                             
@@ -174,12 +180,14 @@ def give_reward(reference_code):
                                 
                                 for k, v in customer_transaction.entitled_voucher_summary.items():
                                     
-                                    voucher_details = MerchantVoucher.fetch(v.get('voucher_key'))
+                                    #voucher_details = MerchantVoucher.fetch(v.get('voucher_key'))
+                                    voucher_details = MerchantVoucher.fetch(v.get('key'))
                                     
                                     customer_entitled_voucher_list.append({
+                                                                            'key'               : k,
                                                                             'label'             : voucher_details.label,
                                                                             'image_url'         : voucher_details.image_public_url,
-                                                                            'amount'            : v.get('amount'),
+                                                                            'count'             : v.get('amount'),
                                                                             'effective_date'    : v.get('effective_date'),
                                                                             'expiry_date'       : v.get('expiry_date'), 
                                                                         })
@@ -205,7 +213,7 @@ def give_reward(reference_code):
     else:
         return create_rest_message('Reference code is required', status_code=StatusCode.BAD_REQUEST)
     
-@reward_api_bp.route('/<reference_code>/redeem', methods=['POST'])
+@reward_api_bp.route('/reference-code/<reference_code>/redeem', methods=['POST'])
 @auth_token_required
 @outlet_key_required
 def redeem_reward(reference_code):
@@ -304,3 +312,190 @@ def redeem_reward(reference_code):
         
     else:
         return create_rest_message('Reference code is required', status_code=StatusCode.BAD_REQUEST)   
+    
+@reward_api_bp.route('/reference-code/<reference_code>/prepaid-topup', methods=['POST'])
+@auth_token_required
+@outlet_key_required
+def prepaid_topup(reference_code):
+    
+    logger.debug('reference_code=%s', reference_code)
+    
+    if is_not_empty(reference_code):  
+        prepaid_topup_data_in_json   = request.get_json()
+          
+        logger.debug('prepaid_topup_data_in_json=%s', prepaid_topup_data_in_json)
+        
+        prepaid_topup_form      = PrepaidTopupForm(ImmutableMultiDict(prepaid_topup_data_in_json))
+        
+        
+        
+        
+        if prepaid_topup_form.validate():
+            
+            prepaid_program_key     = prepaid_topup_form.prepaid_program_key.data
+            topup_amount            = float(prepaid_topup_form.topup_amount.data)
+            invoice_id              = prepaid_topup_form.invoice_id.data
+            remarks                 = prepaid_topup_form.remarks.data
+            merchant_username       = get_logged_in_api_username()
+            customer_transaction    = None    
+            prepaid_summary         = {}
+            
+            db_client = create_db_client(caller_info="prepaid_topup")
+            with db_client.context():
+                topup_outlet            = Outlet.fetch(request.headers.get('x-outlet-key'))
+                merchant_acct           = topup_outlet.merchant_acct_entity
+                customer_acct           = Customer.get_by_reference_code(reference_code, merchant_acct)
+                prepaid_program         = PrepaidSettings.fetch(prepaid_program_key)
+                topup_by_merchant_user  = MerchantUser.get_by_username(merchant_username)
+                
+                if customer_acct and prepaid_program:
+                    
+                    (customer_transaction, prepaid_summary) = create_topup_prepaid_transaction(customer_acct, prepaid_program, 
+                                                                                                topup_amount=topup_amount, 
+                                                                                                topup_outlet=topup_outlet, 
+                                                                                                topup_by=topup_by_merchant_user, 
+                                                                                                invoice_id=invoice_id, 
+                                                                                                remarks = remarks,
+                                                                                                system_remarks = 'Topup Prepaid',
+                                                                                                )
+                    
+                    
+            if customer_transaction is not None:
+                #prepaid_amount              = prepaid_summary.get('amount')
+                #currency_details            = get_merchant_configured_currency_details()
+                #formatted_prepaid_amount    = format_currency_with_currency_label_filter(prepaid_amount, currency_details=currency_details)
+                
+                
+                return create_rest_message('Prepaid have been topup successfully',
+                                            status_code=StatusCode.OK
+                                            )
+            else:
+                return create_rest_message('Failed to topup prepaid', status_code=StatusCode.BAD_REQUEST)
+                
+        else:
+            error_message = prepaid_topup_form.create_rest_return_error_message()
+            
+            return create_rest_message(error_message, status_code=StatusCode.BAD_REQUEST)
+        
+@reward_api_bp.route('/reference-code/<reference_code>/prepaid-redeem', methods=['POST'])
+@auth_token_required
+@outlet_key_required
+def prepaid_redeem(reference_code):
+    
+    logger.debug('reference_code=%s', reference_code)
+    
+    if is_not_empty(reference_code):  
+        prepaid_redeem_data_in_json   = request.get_json()
+          
+        logger.debug('prepaid_redeem_data_in_json=%s', prepaid_redeem_data_in_json)
+        
+        prepaid_redeem_form      = PrepaidRedeemForm(ImmutableMultiDict(prepaid_redeem_data_in_json))
+        
+        
+        
+        
+        if prepaid_redeem_form.validate():
+            
+            redeem_amount           = float(prepaid_redeem_form.redeem_amount.data)
+            invoice_id              = prepaid_redeem_form.invoice_id.data
+            remarks                 = prepaid_redeem_form.remarks.data
+            merchant_username       = get_logged_in_api_username()
+            
+            redemption_details      = None
+            
+            db_client = create_db_client(caller_info="prepaid_redeem")
+            with db_client.context():
+                redeemed_outlet         = Outlet.fetch(request.headers.get('x-outlet-key'))
+                merchant_acct           = redeemed_outlet.merchant_acct_entity
+                customer_acct           = Customer.get_by_reference_code(reference_code, merchant_acct)
+                redeem_by_merchant_user = MerchantUser.get_by_username(merchant_username)
+                
+                if customer_acct:
+                    redemption_details  = redeem_reward_transaction(customer_acct,  
+                                                                redeem_outlet               = redeemed_outlet,
+                                                                reward_format               = program_conf.REWARD_FORMAT_PREPAID,
+                                                                reward_amount               = redeem_amount,
+                                                                invoice_id                  = invoice_id,
+                                                                remarks                     = remarks,
+                                                                redeemed_by                 = redeem_by_merchant_user, 
+                                                                redeemed_datetime           = datetime.utcnow(), 
+                                                                )
+                    
+                    logger.debug('redemption_details=%s', redemption_details)
+                    
+                    
+            if redemption_details is not None:
+                
+                
+                return create_rest_message('Prepaid have been redeemed successfully',
+                                            status_code=StatusCode.OK
+                                            )
+            else:
+                return create_rest_message('Failed to redeem prepaid', status_code=StatusCode.BAD_REQUEST)
+                
+        else:
+            error_message = prepaid_redeem_form.create_rest_return_error_message()
+            
+            return create_rest_message(error_message, status_code=StatusCode.BAD_REQUEST)
+        
+@reward_api_bp.route('/reference-code/<reference_code>/point-redeem', methods=['POST'])
+@auth_token_required
+@outlet_key_required
+def point_redeem(reference_code):
+    
+    logger.debug('reference_code=%s', reference_code)
+    
+    if is_not_empty(reference_code):  
+        point_redeem_data_in_json   = request.get_json()
+          
+        logger.debug('point_redeem_data_in_json=%s', point_redeem_data_in_json)
+        
+        point_redeem_form      = PointRedeemForm(ImmutableMultiDict(point_redeem_data_in_json))
+        
+        
+        
+        
+        if point_redeem_form.validate():
+            
+            redeem_amount           = float(point_redeem_form.redeem_amount.data)
+            invoice_id              = point_redeem_form.invoice_id.data
+            remarks                 = point_redeem_form.remarks.data
+            merchant_username       = get_logged_in_api_username()
+            
+            redemption_details      = None
+            
+            db_client = create_db_client(caller_info="prepaid_redeem")
+            with db_client.context():
+                redeemed_outlet         = Outlet.fetch(request.headers.get('x-outlet-key'))
+                merchant_acct           = redeemed_outlet.merchant_acct_entity
+                customer_acct           = Customer.get_by_reference_code(reference_code, merchant_acct)
+                redeem_by_merchant_user = MerchantUser.get_by_username(merchant_username)
+                
+                if customer_acct:
+                    redemption_details  = redeem_reward_transaction(customer_acct,  
+                                                                redeem_outlet               = redeemed_outlet,
+                                                                reward_format               = program_conf.REWARD_FORMAT_POINT,
+                                                                reward_amount               = redeem_amount,
+                                                                invoice_id                  = invoice_id,
+                                                                remarks                     = remarks,
+                                                                redeemed_by                 = redeem_by_merchant_user, 
+                                                                redeemed_datetime           = datetime.utcnow(), 
+                                                                )
+                    
+                    logger.debug('redemption_details=%s', redemption_details)
+                    
+                    
+            if redemption_details is not None:
+                
+                
+                return create_rest_message('Point have been redeemed successfully',
+                                            status_code=StatusCode.OK
+                                            )
+            else:
+                return create_rest_message('Failed to redeem point', status_code=StatusCode.BAD_REQUEST)
+                
+        else:
+            error_message = point_redeem_form.create_rest_return_error_message()
+            
+            return create_rest_message(error_message, status_code=StatusCode.BAD_REQUEST)        
+                
